syntax = "proto3";

import "unittest_issue6936_a.proto";
import "unittest_issue6936_b.proto";

package unittest_issues;

option csharp_namespace = "UnitTest.Issues.TestProtos";

// This file is used as part of a unit test for issue 6936
// We don't need to use it, we just have to load it at runtime

message Bar {
    option (opt) = "bar";
    Foo foo = 1;
}