// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

syntax = "proto2";

package benchmarks.google_message3;

option cc_enable_arenas = true;
option java_package = "com.google.protobuf.benchmarks";

message Message11018 {}

message Message10800 {
  optional string field10808 = 1;
  optional int64 field10809 = 2;
  optional bool field10810 = 3;
  optional float field10811 = 4;
}

message Message10802 {}

message Message10748 {
  optional string field10750 = 1;
  optional int32 field10751 = 2;
  optional int32 field10752 = 3;
  optional int32 field10753 = 4;
}

message Message7966 {
  optional string field7969 = 1;
  optional bool field7970 = 2;
}

message Message708 {
  optional .benchmarks.google_message3.Message741 field823 = 1;
  repeated string field824 = 6;
  optional string field825 = 2;
  optional string field826 = 3;
  repeated string field827 = 4;
  repeated string field828 = 5;
}

message Message8942 {}

message Message11011 {
  required bytes field11752 = 1;
  required bytes field11753 = 2;
}

message UnusedEmptyMessage {}

message Message741 {
  repeated string field936 = 1;
}
