// Protocol Buffers - Google's data interchange format
// Copyright 2008 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

syntax = "proto3";

package jspb.test;

import "testbinary.proto";

message TestProto3 {
  int32 singular_int32 = 1;
  int64 singular_int64 = 2;
  uint32 singular_uint32 = 3;
  uint64 singular_uint64 = 4;
  sint32 singular_sint32 = 5;
  sint64 singular_sint64 = 6;
  fixed32 singular_fixed32 = 7;
  fixed64 singular_fixed64 = 8;
  sfixed32 singular_sfixed32 = 9;
  sfixed64 singular_sfixed64 = 10;
  float singular_float = 11;
  double singular_double = 12;
  bool singular_bool = 13;
  string singular_string = 14;
  bytes singular_bytes = 15;

  ForeignMessage singular_foreign_message = 19;
  Proto3Enum singular_foreign_enum = 22;

  optional int32 optional_int32 = 121;
  optional int64 optional_int64 = 122;
  optional uint32 optional_uint32 = 123;
  optional uint64 optional_uint64 = 124;
  optional sint32 optional_sint32 = 125;
  optional sint64 optional_sint64 = 126;
  optional fixed32 optional_fixed32 = 127;
  optional fixed64 optional_fixed64 = 128;
  optional sfixed32 optional_sfixed32 = 129;
  optional sfixed64 optional_sfixed64 = 130;
  optional float optional_float = 131;
  optional double optional_double = 132;
  optional bool optional_bool = 133;
  optional string optional_string = 134;
  optional bytes optional_bytes = 135;

  optional ForeignMessage optional_foreign_message = 136;
  optional Proto3Enum optional_foreign_enum = 137;

  repeated int32 repeated_int32 = 31;
  repeated int64 repeated_int64 = 32;
  repeated uint32 repeated_uint32 = 33;
  repeated uint64 repeated_uint64 = 34;
  repeated sint32 repeated_sint32 = 35;
  repeated sint64 repeated_sint64 = 36;
  repeated fixed32 repeated_fixed32 = 37;
  repeated fixed64 repeated_fixed64 = 38;
  repeated sfixed32 repeated_sfixed32 = 39;
  repeated sfixed64 repeated_sfixed64 = 40;
  repeated float repeated_float = 41;
  repeated double repeated_double = 42;
  repeated bool repeated_bool = 43;
  repeated string repeated_string = 44;
  repeated bytes repeated_bytes = 45;

  repeated ForeignMessage repeated_foreign_message = 49;
  repeated Proto3Enum repeated_foreign_enum = 52;

  oneof oneof_field {
    uint32 oneof_uint32 = 111;
    ForeignMessage oneof_foreign_message = 112;
    string oneof_string = 113;
    bytes oneof_bytes = 114;
  }
}

enum Proto3Enum {
  PROTO3_FOO = 0;
  PROTO3_BAR = 1;
  PROTO3_BAZ = 2;
  MSG_PROTO3_BAH = 3;
}
